<?php
/*
 * Copyright (c) 2014 - Copyright holders CIRSFID and Department of
 * Computer Science and Engineering of the University of Bologna
 *
 * Authors:
 * Monica Palmirani – CIRSFID of the University of Bologna
 * Fabio Vitali – Department of Computer Science and Engineering of the University of Bologna
 * Luca Cervone – CIRSFID of the University of Bologna
 *
 * Permission is hereby granted to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The Software can be used by anyone for purposes without commercial gain,
 * including scientific, individual, and charity purposes. If it is used
 * for purposes having commercial gains, an agreement with the copyright
 * holders is required. The above copyright notice and this permission
 * notice shall be included in all copies or substantial portions of the
 * Software.
 *
 * Except as contained in this notice, the name(s) of the above copyright
 * holders and authors shall not be used in advertising or otherwise to
 * promote the sale, use or other dealings in this Software without prior
 * written authorization.
 *
 * The end-user documentation included with the redistribution, if any,
 * must include the following acknowledgment: "This product includes
 * software developed by University of Bologna (CIRSFID and Department of
 * Computer Science and Engineering) and its authors (Monica Palmirani,
 * Fabio Vitali, Luca Cervone)", in the same place and form as other
 * third-party acknowledgments. Alternatively, this acknowledgment may
 * appear in the software itself, in the same form and location as other
 * such third-party acknowledgments.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
// essential for huge PCREs
ini_set("pcre.backtrack_limit", "23001337");
ini_set("pcre.recursion_limit", "23001337");

require_once (dirname(__FILE__) . "/../utils.php");

class AuthorityParser {

	public $lang, $docType;
	private $parserRules = array();

	public function __construct($lang, $docType) {
		$this -> lang = $lang;
		$this -> docType = $docType;
		$this -> dirName = dirname(__FILE__);

		$this -> loadConfiguration();
	}

	public function parse($content, $jsonOutput = FALSE) {
		$return = array();
		if ($this -> lang && $this -> docType && !empty($this -> parserRules)) {
			
			$rules = $this -> parserRules['rules'];
			foreach ($rules as $r) {

				$resolved = resolveRegex($this -> parserRules[$r], $this -> parserRules, $this -> lang, $this -> docType, $this -> dirName);
				$success = preg_match_all($resolved['value'], $content, $result, PREG_OFFSET_CAPTURE);
				
				//print_r($resolved["value"]);
				
				if ($success) {
					
					//print_r($result);

					for ($i = 0; $i < $success; $i++) {

						if (array_key_exists("authority", $result)) {
							$match = $result["authority"][$i][0];
							$aOffset = $result["authority"][$i][1];
							$totalMatch = $result[0][$i][0];
							$offset = $result[0][$i][1];
							$entry = Array("authority" => $match, "start" => $offset, "value" => $totalMatch, "end" => $offset + strlen($totalMatch), "aStart" => $aOffset, "aEnd" => $aOffset + strlen($match));
					    }

						if (array_key_exists("signature", $result)) {
							if (is_array($result["signature"])) {
								$match = trim(trim($result["signature"][$i][0], '.'));
								$offset = $result["signature"][$i][1];
								$entry["sStart"] = $offset;
								$entry["sEnd"] = $offset + strlen($match);
								$entry["signature"] = $match;
								if (array_key_exists("name", $result))
									$entry["name"] = trim(trim($result["name"][$i][0], '.'));
								if (array_key_exists("surname", $result))
									$entry["surname"] = trim(trim($result["surname"][$i][0], '.'));
								if (array_key_exists("keeper", $result))
									$entry["type"] = 'keeper';
								if (array_key_exists("subscription", $result))
									$entry["type"] = 'subscription';

							}
						}
						
						if (array_key_exists("heu_signature", $result)) {
							if (is_array($result["heu_signature"])) {
								$match = trim(trim($result["heu_signature"][$i][0], '.'));
								$offset = $result["heu_signature"][$i][1];
								$entry["sStart"] = $offset;
								$entry["sEnd"] = $offset + strlen($match);
								$entry["signature"] = $match;
								if (array_key_exists("n", $result))
									$entry["name"] = 	trim(trim($result["n"][$i][0], '.'));
								if (array_key_exists("s", $result))
									$entry["surname"] = trim(trim($result["s"][$i][0], '.'));
							}
						}

						$return[] = $entry;
					}
				}

			}

		} else {
			$return = Array('success' => FALSE);
		}

		$ret = array("response" => $return);
		if ($jsonOutput) {
			return json_encode($ret);
		} else {
			return $ret;
		}
	}

	public function loadConfiguration() {
		$this -> parserRules = importParserConfiguration($this -> lang, $this -> docType, $this -> dirName);
	}

}
?>